
#    MIDAS Access Client  for MessageLogger service

global env
source [file join $env(MIDASBASE) SOAP tcl MIDASAccessClient.tcl]    ;#   load default support procedures

#    obtain Message Logger Server host and port using environment variables (if they exist) or 
#    defaults  -  egmsg:8015

global MessageLoggerHost MessageLoggerPort

if {[info exists env(MessageLoggerHost)]} then {set MessageLoggerHost $env(MessageLoggerHost)} \
    else {set MessageLoggerHost egmsg}
if {[info exists env(MessageLoggerPort)]} then {set MessageLoggerPort $env(MessageLoggerPort)} \
    else {set MessageLoggerPort 8015}

#    The service URL is  egmsg:8015/MessageLogger

variable uri "urn:MessageLogger"
variable proxy "http://$MessageLoggerHost:$MessageLoggerPort/MessageLogger"


 global tcl_platform
    switch $tcl_platform(os) {
       "Windows NT" {
           package require registry
           global env
           set env(ComputerName) [registry get HKEY_LOCAL_MACHINE\\SYSTEM\\CurrentControlSet\\Control\\ComputerName\\ActiveComputerName ComputerName]
       }
       SunOS  -
       Linux  {
           global env
           set env(ComputerName) [exec uname -n]
       }
    }

#    create and configure the SOAP method

SOAP::create MessageLogger__DefineMessage -proxy $proxy -uri $uri -timeout 5000 -version 1.2 \
     -name DefineMessage \
     -params {class int level int source string message string} \
     -wrapProc  ::SOAP::MIDAS_SOAPAccessClient_Request \
     -parseProc ::SOAP::MIDAS_SOAPAccessClient_Reply

#    define a Tcl procedure which has some (optional) arguments

#    format of Tcl procedure call to define a message is:-        DefineMessage [-c class] [-l level] [-s source] message

proc DefineMessage  {args} {
    global errorCode errorInfo env
    
    set Class 1       ;# define some default values
    set Level 3

    set Source $env(ComputerName)
    set Message ""

    set l [llength $args]
    if {$l < 1}  {return 0}      ;# no message
    if {$l == 1} {
        set Message "$args"
    } else {
        for {set i 0} {$i < $l} {incr i} {
            switch -- [lindex $args $i] {
                -c   {incr i; set Class [lindex $args $i]}
                -l   {incr i; set Level [lindex $args $i]}
                -s   {incr i; set Source [lindex $args $i]}
                default {set Message [lreplace $args 0 [expr $i-1]]}
            }
            if {$Message != ""} {break}
        }
    }

   set z [catch {MessageLogger__DefineMessage $Class $Level "$Source" "$Message"} m]  ;# use SOAP method to define the message

   if {$z != 0} {puts stderr "Error in DefineMessage: $m\n$errorCode\n$errorInfo"}
}


package provide MessageLoggerClient 1.0

