
#    these procedures implement the "endrun" and "newrun" functions
#    they can be included in any task but normally it is assumed
#    they would be connected to button in the main SYS control window

set newrun_active 0
set no_newrun 0


proc starttapetask {} {
#    Check if it is necessary to pre-start tape server interface task

    set started [inform tape inquire started]
    if {$started == 0} then {
      send tape make-frame
      send tape close-frame
    }
}

proc do_endrun {datastream} {
   global tapestate nstream BlkCount OPENmode OPENlbls OPENreclen OPENblklen OPENfile OPENvol OPENstate
   global no_newrun
   set-footer "Starting Endrun procedure"
   set working 0
   set no_newrun 0
   starttapetask
#    first action is to STOP the tape server if not in STOPped state
   set tapestate [send tape read-egstate]
   if {$tapestate != 1} then {send tape click-egstate 0}
#    next find drives currently inuse (associated)
   set tapeassoc [send tape find-assoc-strings $datastream]
   set drivelist [split $tapeassoc +]
   set closelist ""
   foreach item $drivelist {
#    for each drive find alias and current state
      foreach drive $item {
         set alias [send tape aliases-from-drives $drive]
         set fstatus [send tape query-file-status $alias]
         set state [second $fstatus]
         set BlkCount($drive) [lindex $fstatus 15]
#    also save information which will be needed for a "newrun"
         set OPENstate($drive) 0
         set OPENmode($drive) [lindex $fstatus 7]
         set OPENlbls($drive) [lindex $fstatus 8]
         set OPENreclen($drive) [lindex $fstatus 9]
         set OPENblklen($drive) [lindex $fstatus 10]
         set OPENvol($drive) [lindex $fstatus 5]
         set OPENfile($drive) [first [split [lindex $fstatus 6] /]]
         if {$state == 5} then {
#    this drive is open - so issue a close
            set OPENstate($drive) 1
            append closelist " $drive"
            set-footer "Closing $closelist"
            send tape set alias $alias
            send tape close-file
            incr working
         } else {set no_newrun 1}
      }
   }
#    all drives are now being closed 
  if {$working == 0} then {
      resume_endrun1 $datastream
  } else {
      after 2000 resume_endrun1 $datastream
  }
}

proc resume_endrun1 {datastream} {
   global BlkCount OPENstate nstream newrun_active no_newrun
#    next find drives currently inuse (associated)
   set tapeassoc [send tape find-assoc-strings $datastream]
   set drivelist [split $tapeassoc +]
   foreach item $drivelist {
#    for each drive find alias and current state
      foreach drive $item {
         set alias [send tape aliases-from-drives $drive]
         set st [send tape query-file-status $alias]
         set state [second $st]
         if {$state == 6} then {
#    this drive is closing - so wait for completion (try agin in 5 seconds)
            set-footer "waiting for drive $drive to complete close operation"
            after 5000 resume_endrun1 $datastream
            return
         } 
         if {$OPENstate($drive) != 0} then {insert-log "sys: [date] file closed on drive $drive after $BlkCount($drive) blocks"}
      }
   }
#    all drives of interest now closed
   set-footer "Endrun task complete at [date]"
   send tape redisplay
   if {$newrun_active == 1} then {
     if {$no_newrun == 0} then {resume_newrun1 $datastream} \
       else {midas-warning "Cannot complete Newrun procedure because some tapes were not initially in the OPEN state" }
   }
}


proc do_newrun {datastream} {
  global newrun_active tsautomode
#    first perform "endrun" actions
#    switch off any auto associations actions since we intend to
#    revert to same association state
   starttapetask
   set tsautomode [send tape inquire tsautomode]
   send tape set tsautomode 0
   set newrun_active 1
   do_endrun $datastream
}

proc resume_newrun1 {datastream} {
   global tapestate OPENmode OPENlbls OPENreclen OPENblklen OPENfile
   set-footer "Starting Newrun procedure"
#    next find drives currently inuse (associated)
   set tapeassoc [send tape find-assoc-strings $datastream]
   set drivelist [split $tapeassoc +]
   set openlist ""
   foreach item $drivelist {
#    for each drive find alias and current state
      foreach drive $item {
         set alias [send tape aliases-from-drives $drive]
         set state [second [send tape query-file-status $alias]]
         if {$state == 3} then {
#    this drive is mounted (closed) - so issue a open
            append openlist " $drive"
            set-footer "Opening $openlist"
            send tape set alias $alias
            set f $OPENfile($drive)
            set froot [first [split $f 0123456789]]
            set findex [string range $f [string length $froot] end]
            if {$findex == ""} then {set findex 0}
            set f $froot[incr findex]
            send tape set filename $f
            set OPENfile($drive) $f
            send tape open-file $OPENmode($drive) $OPENlbls($drive) $OPENreclen($drive) $OPENblklen($drive)
         } 
      }
   }
#    all drives are now being opened
   after 2000 resume_newrun2 $datastream
}

proc resume_newrun2 {datastream} {
   global env newrun_active tsautomode OPENfile OPENvol
#    find drives currently inuse (associated)
   set tapeassoc [send tape find-assoc-strings $datastream]
   set drivelist [split $tapeassoc +]
   foreach item $drivelist {
#    for each drive find alias and current state
      foreach drive $item {
         set alias [send tape aliases-from-drives $drive]
         set state [second [send tape query-file-status $alias]]
         if {$state == 4} then {
#    this drive is opening - so wait for completion (try agin in 5 seconds)
            set-footer "waiting for drive $drive to complete open operation"
            after 5000 resume_newrun2 $datastream
            return
         } 
         insert-log "sys: [date] file $OPENfile($drive) opened on drive $drive"
         insert-log "sys: [date] Volume on $drive is $OPENvol($drive), file is $OPENfile($drive)"

      }
   }
#    all drives of interest now opened
   if {$env(TCLTS) == "orbx"} then {
#    must write header
      foreach item $drivelist {
         foreach drive $item {
            set alias [send tape aliases-from-drives $drive]
            set-footer "Writing header"
            send tape set alias $alias
            send tape setup-header
            send tape write-header
         }
      }
#    writing header to all drives
    after 2000 resume_newrun3 $datastream
   } else {
#    for RPC server finished
#    restore auto mode
      send tape set tsautomode $tsautomode
      set newrun_active 0
#    all drives of interest now opened
      set-footer "Newrun task complete at [date]"
      send tape redisplay
  }
}


proc resume_newrun3 {datastream} {
   global newrun_active tsautomode
#    find drives currently inuse (associated)
   set tapeassoc [send tape find-assoc-strings $datastream]
   set drivelist [split $tapeassoc +]
   foreach item $drivelist {
#    for each drive find alias and current state
      foreach drive $item {
         set alias [send tape aliases-from-drives $drive]
         set state [second [send tape query-file-status $alias]]
         if {$state == 23} then {
#    this drive is writing header - so wait for completion (try agin in 5 seconds)
            set-footer "waiting for drive $drive to complete write header operation"
            after 5000 resume_newrun3 $datastream
            return
         } 
         insert-log "sys: [date] file header written on drive $drive"
      }
   }
#    restore auto mode
   send tape set tsautomode $tsautomode
   set newrun_active 0
#    all drives of interest now opened
   set-footer "Newrun task complete at [date]"
   send tape redisplay
}

